pro lwa_sxh, sscfiles, outdir=outdir, run_time=run_time, $
	    testss=testss, vignette=vignette
;+
;
;   Name: mk_sxh
;
;   Purpose: Generate the SXT X-Ray Histogram Log (SXH) from SSC files
;
;   Input Parameters:
;      sscfiles - the input ssc files, Nominally one per day
;
;   Optional Keyword Parameters:
;	outdir	- The output directory for the file.  Default = $DIR_SXT_SXH
;	/vignette, applies vignette correction to SSC before forming
;		histogram IF the SSC is not already vignette corrected.
;
;   METHOD (~same as mk_sxl) :
;	Six histograms are saved for each SSC image.  If the data is
;	read out into the variable SXL, then the following applies:
;		sxh.hist(*,0) = full image (all pixels)
;		sxh.hist(*,1) = all pixels inside 0.86 of the solar
;				radius
;		sxh.hist(*,2) = all pixels outside of 1.05 of the solar
;				radius
;		sxh.hist(*,[3,4,5]) = as above but for negative values
;				in the image
;	Each of the histograms is 539 elements and is the histogram of
;          alog10( (data+1)>1.) with binsize = .01
;
;	NOTE: The negative-value histograms are included to properly
;		represent the effects of statistical noise in the
;		low-signal parts of the image.
;	
;	The histogram of the whole-sun image can be derived by:
;		ann_hist = sxh.hist(*,0) - sxh.hist(*,3)
;
;	The histogram of the "annulus" (pixels around the very limb
;	of the sun) can be derived by:
;		ann_hist = (sxh.hist(*,0) - sxh.hist(*,1) - sxh.hist(*,2))
;			- (sxh.hist(*,3) - sxh.hist(*,4) - sxh.hist(*,5))
;
;   HISTORY
;	23-March-1999 - S.L.Freeland MK_SXH Written  from mdm mk_sxl
;                       using 'new SSW technology' and write_genxcat  
;        6-apr-1999 - S.L.Freeland - add /VIGNETTE keyword and function
;
;       Note: this file has simplified structure and includes all TAGS
;             previously split between sxl roadmap and sxl.HISTR
;
;	4-May-99, LWA	Increased log10(intensity) range to 5.39 and
;			handled negative values as an inverse image
;
;       7-May-99, S.L.Freeland - lwa_sxh->mk_sxh, made it version 1.1
;                                change structure def to match newdim (540)
;
;	15-Jun-00, LWA  Corrected file size of sxh_struct.his to 539 from 540.
;			Changed SXL to SXH in header.
;			Added explanation of /vignette keyword to header.
;			Changed version to 1.2.
;
;	18-Jul-00, LWA  Deleted unused qstop keyword.
;
;	19-Jul-00, LWA	Hacked a fix for file naming section and requirement
;			for outdir.  Now default to local directory.
;-
vignette=keyword_set(vignette)
help,vignette

common mk_sxh_blk, sxh_struct  

if not data_chk(sscfiles,/string) then begin
   box_message,['Need SSC file list...',$
		'IDL> mk_sxh,sscfiles [,outdir=outdir]']
   return
endif  

start_time = systime(1)
run_time = 0.
;
progVerNo = 1.200*1000
progName = 'MK_SXH.PRO'
;

;if not data_chk(sxh_struct,/struct) then begin
   sxt_struct, sxt_version=sxt_version, sxt_xray2log=sxt_xraylog, sxl_roadmap=sxl_roadmap
   sxhstruct=add_tag(sxl_roadmap, '', 'SSCFILE')        ; add ssc file name
   sxhstruct=add_tag(sxhstruct, 0l, 'HISTOVER')         ; histogram overflow
   sxhstruct=add_tag(sxhstruct, lonarr(539,6),'HIST')   ; the 'new' record
   sxh_struct=sxhstruct
;endif   

; ---------- where to stick'em -------------
;if not keyword_set(outdir) then outdir=get_logenv('DIR_SXT_SXH')
;if not file_exist(outdir) then begin
;   box_message,['Supply OUTDIR or define $DIR_SXT_SXH']
;   return
;endif   

sscok=where(file_exist(sscfiles),okcnt)
if okcnt eq 0 then begin
   box_message,'None of your input sscfiles exist, returning...'
   return
endif  

sscs=sscfiles(sscok)
sscflags=strsplit(sscs,'_',/last,/tail)
break_file,sscs,ll,pp,ff,ee,vv
sscnames=ff+ee+vv
prefix='sxh'
nx_last=-1
if not keyword_set(testss) then testss=1000000.  ; force all if not testing

for i=0,okcnt-1 do begin                   ; for each file....
   print,i,'  ',sscs(i)
   rd_roadmap, sscs(i), roadmap
   ndatasets = n_elements(roadmap)
   sxh_out = replicate(sxh_struct, ndatasets)
;   catname=concat_dir(outdir,prefix+anytim2weekinfo(roadmap(0))) + '_'+ sscflags(i)
   catname=strip_dirname(sscs(i))
   catname=strsplit(catname,'ssc',/tail)
   catname=[prefix+catname]


   nloop=min([ndatasets,testss])
   for idset=0, nloop-1 do begin       ; for each image....

       rd_xda, sscs(i), idset, index, data, roadmap
       if vignette and index.his.flat_tech eq 0 then begin
;          box_message,'applying vignette correction'
	  data=sxt_off_axis(index,data,/update)
       endif 
       dlog10=safe_log10(data)                             ; positive values
       histlimit=where(dlog10 ge 5.390, histover)          ; flag overflow
       dlog10=dlog10<5.390				   ; truncate if req'd
       hist = histogram(dlog10, min=0,max=5.390,bin=.010)  ; make histogram
       nlog10=safe_log10(-data)                            ; negative values
       nhist = histogram(nlog10, min=0,max=5.390,bin=.010) ; make histogram
       
       ; ------- the following code direct from mk_sxl -------
       nx = data_chk(data,/nx)
       ny = data_chk(data,/ny)
       if (nx ne nx_last) then begin	;only remake this array if necessary
	   x = findgen(nx,ny) mod nx
	   y = findgen(nx,ny) / nx
   	   nx_last = nx
       end
       cen = sxt_cen(index)/(2^gt_res(index))
       rarr = sqrt( (x-cen(0))^2 + (y-cen(1))^2 ) * gt_pix_size(index)
       r = get_rb0p(index)
       r = float(r(0))	;radius in arcseconds
       rarr = rarr/r
;      -------------------------------------------------
       
       histinner = lonarr(539)
       nhistinner = lonarr(539)
       ss = where(rarr lt 0.86,incnt)
       if (incnt gt 0) then histinner= $
          histogram(safe_log10(data(ss)), min=0,max=5.390,bin=.010) 
       if (incnt gt 0) then nhistinner= $
          histogram(safe_log10(-data(ss)), min=0,max=5.390,bin=.010) 

       histouter = lonarr(539)
       nhistouter = lonarr(539)
       ss = where(rarr gt 1.05,outcnt)
       if (outcnt gt 0) then histouter = $
	  histogram(safe_log10(data(ss)), min=0,max=5.390,bin=.010)
       if (outcnt gt 0) then nhistouter = $
	  histogram(safe_log10(-data(ss)), min=0,max=5.390,bin=.010)

       avg0 = average(data)
       dev0 = stdev(data)

;      -------- assign histogram values -----------

;       index.sxt.extra   = histover           ; #pixels gt 10.^5.39/sec
       
       sxh_out.hist(*,0) = hist
       sxh_out.hist(*,1) = histinner
       sxh_out.hist(*,2) = histouter
       sxh_out.hist(*,3) = nhist
       sxh_out.hist(*,4) = nhistinner
       sxh_out.hist(*,5) = nhistouter       
;
       sxh_out.sscfile=sscnames(i)

;      -------- fill tags from index --------------
       sxh_out(idset).time                    = index.gen.time
       sxh_out(idset).day                     = index.gen.day
       sxh_out(idset).dp_mode                 = index.gen.dp_mode
       sxh_out(idset).dp_rate                 = index.gen.dp_rate
       sxh_out(idset).pfi_ffi                 = index.sxt.pfi_ffi
       sxh_out(idset).periph                  = index.sxt.periph
       sxh_out(idset).explevmode              = index.sxt.explevmode
       sxh_out(idset).imgparam                = index.sxt.imgparam
       sxh_out(idset).Seq_Num                 = index.sxt.Seq_Num
       sxh_out(idset).seq_tab_serno           = index.sxt.seq_tab_serno

;      --------- fill calculated image statistics --------
       sxh_out(idset).Img_Avg                 = avg0
       sxh_out(idset).Img_Dev                 = dev0

       sav_sda, catname, index, sxh_out.hist, append= (idset ne 0)    

   endfor     ; --------- end of images loop   ------------------                
   
endfor        ; --------- end of ssc files loop -----------
;

; --------- close up ... -------------------------
end_time = systime(1)
run_time = (end_time-start_time)/60.
print, 'MK_SXH took', run_time, ' minutes to run'
;

end
